@@ -112,7 +112,7 @@ context 'Invoker' do
     assert_match(/input file and output file cannot be the same/, invoker.read_error)
   end
 
-  test 'should accept input from named pipe and output to stdout' do
+  test 'should accept input from named pipe and output to stdout', unless: windows? do
     sample_inpath = fixture_path 'sample-pipe.adoc'
     begin
       %x(mkfifo #{sample_inpath})
@@ -126,7 +126,7 @@ context 'Invoker' do
     ensure
       FileUtils.rm_f sample_inpath
     end
-  end unless windows?
+  end
 
   test 'should allow docdir to be specified when input is a string' do
     expected_docdir = fixturedir
@@ -167,7 +167,10 @@ context 'Invoker' do
     begin
       warnings = nil
       redirect_streams do |out, err|
-        invoke_cli_to_buffer(%w(-w -o /dev/null), '-') { $NO_SUCH_VARIABLE || 'text' }
+        invoke_cli_to_buffer(%w(-w -o /dev/null), '-') {
+          A_CONST = 10  
+          A_CONST = 20
+        }
         warnings = err.string
       end
       assert_equal false, $VERBOSE
@@ -670,9 +673,9 @@ context 'Invoker' do
   end
 
   test 'should set eRuby impl if specified' do
-    invoker = invoke_cli_to_buffer %w(--eruby erubis -o /dev/null)
+    invoker = invoke_cli_to_buffer %w(--eruby erubi -o /dev/null)
     doc = invoker.document
-    assert_equal 'erubis', doc.instance_variable_get('@options')[:eruby]
+    assert_equal 'erubi', doc.instance_variable_get('@options')[:eruby]
   end
 
   test 'should force default external encoding to UTF-8' do
@@ -680,7 +683,7 @@ context 'Invoker' do
     # using open3 to work around a bug in JRuby process_manager.rb,
     # which tries to run a gsub on stdout prematurely breaking the test
     # warnings may be issued, so don't assert on stderr
-    stdout_lines = run_command({ 'LANG' => 'US-ASCII' }, %(#{asciidoctor_cmd} -o - --trace #{input_path})) {|out| out.readlines }
+    stdout_lines = run_command(asciidoctor_cmd, '-o', '-', '--trace', input_path, env: { 'LANG' => 'US-ASCII' }) {|out| out.readlines }
     refute_empty stdout_lines
     # NOTE Ruby on Windows runs with a IBM437 encoding by default
     stdout_lines.each {|l| l.force_encoding Encoding::UTF_8 } unless Encoding.default_external == Encoding::UTF_8
@@ -689,7 +692,9 @@ context 'Invoker' do
   end
 
   test 'should force stdio encoding to UTF-8' do
-    result = run_command(%(#{asciidoctor_cmd true, '-E IBM866:IBM866'} -r #{fixture_path 'configure-stdin.rb'} -s -o - -)) {|out| out.read }
+    cmd = asciidoctor_cmd ['-E', 'IBM866:IBM866']
+    # NOTE configure-stdin.rb populates stdin
+    result = run_command(cmd, '-r', (fixture_path 'configure-stdin.rb'), '-s', '-o', '-', '-') {|out| out.read }
     # NOTE Ruby on Windows runs with a IBM437 encoding by default
     result.force_encoding Encoding::UTF_8 unless Encoding.default_external == Encoding::UTF_8
     assert_equal Encoding::UTF_8, result.encoding
@@ -698,8 +703,8 @@ context 'Invoker' do
   end
 
   test 'should not fail to load if call to Dir.home fails' do
-    rubyopt = %(-r #{fixture_path 'undef-dir-home.rb'})
-    result = run_command(%(#{asciidoctor_cmd true, rubyopt} -s -o - #{fixture_path 'basic.adoc'})) {|out| out.read }
+    cmd = asciidoctor_cmd ['-r', (fixture_path 'undef-dir-home.rb')]
+    result = run_command(cmd, '-s', '-o', '-', (fixture_path 'basic.adoc')) {|out| out.read }
     assert_include 'Body content', result
   end
 
@@ -718,7 +723,7 @@ context 'Invoker' do
 
   test 'should show timezone as UTC if system TZ is set to UTC' do
     input_path = fixture_path 'doctime-localtime.adoc'
-    output = run_command({ 'TZ' => 'UTC', 'SOURCE_DATE_EPOCH' => nil }, %(#{asciidoctor_cmd} -d inline -o - -s #{input_path})) {|out| out.read }
+    output = run_command(asciidoctor_cmd, '-d', 'inline', '-o', '-', '-s', input_path, env: { 'TZ' => 'UTC', 'SOURCE_DATE_EPOCH' => nil, 'IGNORE_SOURCE_DATE_EPOCH' => '1' }) {|out| out.read }
     doctime, localtime = output.lines.map(&:chomp)
     assert doctime.end_with?(' UTC')
     assert localtime.end_with?(' UTC')
@@ -726,7 +731,7 @@ context 'Invoker' do
 
   test 'should show timezone as offset if system TZ is not set to UTC' do
     input_path = fixture_path 'doctime-localtime.adoc'
-    output = run_command({ 'TZ' => 'EST+5', 'SOURCE_DATE_EPOCH' => nil }, %(#{asciidoctor_cmd} -d inline -o - -s #{input_path})) {|out| out.read }
+    output = run_command(asciidoctor_cmd, '-d', 'inline', '-o', '-', '-s', input_path, env: { 'TZ' => 'EST+5', 'SOURCE_DATE_EPOCH' => nil, 'IGNORE_SOURCE_DATE_EPOCH' => '1' }) {|out| out.read }
     doctime, localtime = output.lines.map(&:chomp)
     assert doctime.end_with?(' -0500')
     assert localtime.end_with?(' -0500')
