@@ -1,8 +1,5 @@
-# encoding: UTF-8
-unless defined? ASCIIDOCTOR_PROJECT_DIR
-  $: << File.dirname(__FILE__); $:.uniq!
-  require 'test_helper'
-end
+# frozen_string_literal: true
+require_relative 'test_helper'
 
 context 'Logger' do
   MyLogger = Class.new Logger
@@ -80,6 +77,12 @@ context 'Logger' do
       assert_includes err_string, %(asciidoctor: WARNING: this is a call)
       assert_includes err_string, %(asciidoctor: FAILED: it cannot be done)
     end
+
+    test 'NullLogger level is not nil' do
+      logger = Asciidoctor::NullLogger.new
+      refute_nil logger.level
+      assert_equal Logger::WARN, logger.level
+    end
   end
 
   context ':logger API option' do
@@ -87,7 +90,7 @@ context 'Logger' do
       old_logger = Asciidoctor::LoggerManager.logger
       new_logger = MyLogger.new $stdout
       begin
-        Asciidoctor.load 'contents', :logger => new_logger
+        Asciidoctor.load 'contents', logger: new_logger
         assert_same new_logger, Asciidoctor::LoggerManager.logger
       ensure
         Asciidoctor::LoggerManager.logger = old_logger
@@ -98,7 +101,7 @@ context 'Logger' do
       old_logger = Asciidoctor::LoggerManager.logger
       new_logger = MyLogger.new $stdout
       begin
-        Asciidoctor.load_file fixture_path('basic.asciidoc'), :logger => new_logger
+        Asciidoctor.load_file fixture_path('basic.adoc'), logger: new_logger
         assert_same new_logger, Asciidoctor::LoggerManager.logger
       ensure
         Asciidoctor::LoggerManager.logger = old_logger
@@ -109,7 +112,7 @@ context 'Logger' do
       old_logger = Asciidoctor::LoggerManager.logger
       new_logger = MyLogger.new $stdout
       begin
-        Asciidoctor.convert 'contents', :logger => new_logger
+        Asciidoctor.convert 'contents', logger: new_logger
         assert_same new_logger, Asciidoctor::LoggerManager.logger
       ensure
         Asciidoctor::LoggerManager.logger = old_logger
@@ -120,7 +123,7 @@ context 'Logger' do
       old_logger = Asciidoctor::LoggerManager.logger
       new_logger = MyLogger.new $stdout
       begin
-        Asciidoctor.convert_file fixture_path('basic.asciidoc'), :to_file => false, :logger => new_logger
+        Asciidoctor.convert_file fixture_path('basic.adoc'), to_file: false, logger: new_logger
         assert_same new_logger, Asciidoctor::LoggerManager.logger
       ensure
         Asciidoctor::LoggerManager.logger = old_logger
@@ -141,7 +144,7 @@ context 'Logger' do
         include SampleModuleA
       end
       assert_same Asciidoctor::LoggerManager.logger, SampleClassA.new.get_logger
-      assert SampleClassA.private_method_defined? :logger
+      assert SampleClassA.public_method_defined? :logger
     end
 
     test 'including Logging gives static methods on module access to logging infrastructure' do
@@ -164,7 +167,7 @@ context 'Logger' do
       end
 
       assert_same Asciidoctor::LoggerManager.logger, SampleClassC.new.get_logger
-      assert SampleClassC.private_method_defined? :logger
+      assert SampleClassC.public_method_defined? :logger
     end
 
     test 'including Logging gives static methods on class access to logging infrastructure' do
@@ -182,7 +185,7 @@ context 'Logger' do
       class SampleClassE
         include Asciidoctor::Logging
         def create_message cursor
-          message_with_context 'Asciidoctor was here', :source_location => cursor
+          message_with_context 'Asciidoctor was here', source_location: cursor
         end
       end
 
@@ -194,18 +197,18 @@ context 'Logger' do
     end
 
     test 'writes message prefixed with program name and source location to stderr' do
-      input = <<-EOS
-[#first]
-first paragraph
+      input = <<~'EOS'
+      [#first]
+      first paragraph
 
-[#first]
-another first paragraph
+      [#first]
+      another first paragraph
       EOS
       messages = redirect_streams do |_, err|
         convert_string_to_embedded input
-        err.string
+        err.string.chomp
       end
-      assert_equal 'asciidoctor: WARNING: <stdin>: line 5: id assigned to block already in use: first', messages.chomp
+      assert_equal 'asciidoctor: WARNING: <stdin>: line 5: id assigned to block already in use: first', messages
     end
   end
 end
